@@ -1,14 +1,13 @@
 import asyncio
-import functools
 import typing
 
 from starlette.requests import Request
-from starlette.responses import StreamingResponse
-from starlette.types import ASGIApp, ASGIInstance, Receive, Scope, Send
+from starlette.responses import Response, StreamingResponse
+from starlette.types import ASGIApp, Receive, Scope, Send
 
-RequestResponseEndpoint = typing.Callable[[Request], typing.Awaitable[ASGIInstance]]
+RequestResponseEndpoint = typing.Callable[[Request], typing.Awaitable[Response]]
 DispatchFunction = typing.Callable[
-    [Request, RequestResponseEndpoint], typing.Awaitable[ASGIInstance]
+    [Request, RequestResponseEndpoint], typing.Awaitable[Response]
 ]
 
 
@@ -17,24 +16,26 @@ class BaseHTTPMiddleware:
         self.app = app
         self.dispatch_func = self.dispatch if dispatch is None else dispatch
 
-    def __call__(self, scope: Scope) -> ASGIInstance:
+    async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
         if scope["type"] != "http":
-            return self.app(scope)
-        return functools.partial(self.asgi, scope=scope)
+            await self.app(scope, receive, send)
+            return
 
-    async def asgi(self, receive: Receive, send: Send, scope: Scope) -> None:
         request = Request(scope, receive=receive)
         response = await self.dispatch_func(request, self.call_next)
-        await response(receive, send)
+        await response(scope, receive, send)
 
-    async def call_next(self, request: Request) -> ASGIInstance:
-        inner = self.app(dict(request))
+    async def call_next(self, request: Request) -> Response:
         loop = asyncio.get_event_loop()
         queue = asyncio.Queue()  # type: asyncio.Queue
 
+        scope = request.scope
+        receive = request.receive
+        send = queue.put
+
         async def coro() -> None:
             try:
-                await inner(request.receive, queue.put)
+                await self.app(scope, receive, send)
             finally:
                 await queue.put(None)
 
@@ -62,5 +63,5 @@ class BaseHTTPMiddleware:
 
     async def dispatch(
         self, request: Request, call_next: RequestResponseEndpoint
-    ) -> ASGIInstance:
+    ) -> Response:
         raise NotImplementedError()  # pragma: no cover
