@@ -7,7 +7,7 @@ import stat
 import typing
 from email.utils import formatdate
 from mimetypes import guess_type
-from urllib.parse import quote_plus
+from urllib.parse import quote, quote_plus
 
 from starlette.background import BackgroundTask
 from starlette.concurrency import iterate_in_threadpool
@@ -18,8 +18,8 @@ try:
     import aiofiles
     from aiofiles.os import stat as aio_stat
 except ImportError:  # pragma: nocover
-    aiofiles = None  # type: ignore
-    aio_stat = None  # type: ignore
+    aiofiles = None
+    aio_stat = None
 
 try:
     import ujson
@@ -97,20 +97,20 @@ class Response:
         secure: bool = False,
         httponly: bool = False,
     ) -> None:
-        cookie = http.cookies.SimpleCookie()
+        cookie = http.cookies.SimpleCookie()  # type: http.cookies.BaseCookie
         cookie[key] = value
         if max_age is not None:
-            cookie[key]["max-age"] = max_age  # type: ignore
+            cookie[key]["max-age"] = max_age
         if expires is not None:
-            cookie[key]["expires"] = expires  # type: ignore
+            cookie[key]["expires"] = expires
         if path is not None:
             cookie[key]["path"] = path
         if domain is not None:
             cookie[key]["domain"] = domain
         if secure:
-            cookie[key]["secure"] = True  # type: ignore
+            cookie[key]["secure"] = True
         if httponly:
-            cookie[key]["httponly"] = True  # type: ignore
+            cookie[key]["httponly"] = True
         cookie_val = cookie.output(header="").strip()
         self.raw_headers.append((b"set-cookie", cookie_val.encode("latin-1")))
 
@@ -156,6 +156,7 @@ class UJSONResponse(JSONResponse):
     media_type = "application/json"
 
     def render(self, content: typing.Any) -> bytes:
+        assert ujson is not None, "ujson must be installed to use UJSONResponse"
         return ujson.dumps(content, ensure_ascii=False).encode("utf-8")
 
 
@@ -228,7 +229,13 @@ class FileResponse(Response):
         self.background = background
         self.init_headers(headers)
         if self.filename is not None:
-            content_disposition = 'attachment; filename="{}"'.format(self.filename)
+            content_disposition_filename = quote(self.filename)
+            if content_disposition_filename != self.filename:
+                content_disposition = "attachment; filename*=utf-8''{}".format(
+                    content_disposition_filename
+                )
+            else:
+                content_disposition = 'attachment; filename="{}"'.format(self.filename)
             self.headers.setdefault("content-disposition", content_disposition)
         self.stat_result = stat_result
         if stat_result is not None:
